cd "c:/mijn documenten/projecten/stata/proprcspline/1.2.3/bench"
discard
use nlsw88, clear
gen marst = cond(married, 1, ///
            cond(never_married, 2, 3)) if !missing(married, never_married)
label define marst 1 "married" ///
                   2 "never married" ///
                   3 "divorced/widowed"
label value marst marst

gen black = race == 2 if race < .
label define black 1 "black" ///
                   0 "non-black"
label value black black


proprcspline marst grade, xlab(0(5)15)  by(black, note("")) showknots stub(sp) gen(pr)

assert         r(N_knots) == 5

tempname T_knots 
mat `T_knots' = J(1,5,0)
mat `T_knots'[1,1] =                  9
mat `T_knots'[1,2] =                 12
mat `T_knots'[1,3] =                 12
mat `T_knots'[1,4] =                 14
mat `T_knots'[1,5] =                 18
tempname C_knots
matrix `C_knots' = r(knots)
assert mreldif( `C_knots' , `T_knots' ) < 1E-8
_assert_streq `"`: rowfullnames `C_knots''"' `"grade"'
_assert_streq `"`: colfullnames `C_knots''"' `"knot1 knot2 knot3 knot4 knot5"'
mat drop `C_knots' `T_knots'

sum sp1
assert         r(sum)   == 29394
assert         r(max)   == 18
assert         r(min)   == 0
assert reldif( r(sd)     , 2.521246094581113 ) <  1E-8
assert reldif( r(Var)    , 6.356681869440517 ) <  1E-8
assert reldif( r(mean)   , 13.09893048128342 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp2
assert reldif( r(sum)    , 3711.796357750893 ) <  1E-8
assert reldif( r(max)    , 7.222222328186035 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , 2.167901426346746 ) <  1E-8
assert reldif( r(Var)    , 4.699796594356254 ) <  1E-8
assert reldif( r(mean)   , 1.654098198641218 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp3
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4395329954959572 ) <  1E-8
assert reldif( r(Var)    , .1931892541296491 ) <  1E-8
assert reldif( r(mean)   , .2451475491338873 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp4
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4395329954959572 ) <  1E-8
assert reldif( r(Var)    , .1931892541296491 ) <  1E-8
assert reldif( r(mean)   , .2451475491338873 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr1
assert reldif( r(sum)    , 1440.999989017844 ) <  1E-8
assert reldif( r(max)    , .7185714840888977 ) <  1E-8
assert reldif( r(min)    , .2270296663045883 ) <  1E-8
assert reldif( r(sd)     , .107443531986806  ) <  1E-8
assert reldif( r(Var)    , .0115441125657998 ) <  1E-8
assert reldif( r(mean)   , .6421568578510892 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr2
assert reldif( r(sum)    , 233.9999980814755 ) <  1E-8
assert reldif( r(max)    , .2146780043840408 ) <  1E-8
assert reldif( r(min)    , .0493761226534843 ) <  1E-8
assert reldif( r(sd)     , .0614711531946941 ) <  1E-8
assert reldif( r(Var)    , .0037787026750855 ) <  1E-8
assert reldif( r(mean)   , .1042780740113527 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr3
assert reldif( r(sum)    , 569.000004529953  ) <  1E-8
assert reldif( r(max)    , .5826510787010193 ) <  1E-8
assert reldif( r(min)    , .1576938182115555 ) <  1E-8
assert reldif( r(sd)     , .0640098676234907 ) <  1E-8
assert reldif( r(Var)    , .0040972631531768 ) <  1E-8
assert reldif( r(mean)   , .2535650644072874 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244


// check if by option gets the same predicted proportions for blacks as restricting the sample
keep if black == 1 
proprcspline marst grade, xlab(0(5)15) gen(pr_black) knots(9 12 14 18)

assert pr1 == pr_black1
assert pr2 == pr_black2
assert pr3 == pr_black3

// the bottom category used to run over the entire x-axis, ignoring the if-condition
// this should now be fixed

use nlsw88, clear
gen marst = cond(married, 1, ///
            cond(never_married, 2, 3)) if !missing(married, never_married)
label define marst 1 "married" ///
                   2 "never married" ///
                   3 "divorced/widowed"
label value marst marst

gen black = race == 2 if race < .
label define black 1 "black" ///
                   0 "non-black"
label value black black

proprcspline marst grade if grade > 3, xlab(0(5)15)  by(black, note("")) showknots stub(sp) gen(pr)

assert         r(N_knots) == 5

tempname T_knots 
mat `T_knots' = J(1,5,0)
mat `T_knots'[1,1] =                  9
mat `T_knots'[1,2] =                 12
mat `T_knots'[1,3] =                 12
mat `T_knots'[1,4] =                 14
mat `T_knots'[1,5] =                 18
tempname C_knots
matrix `C_knots' = r(knots)
assert mreldif( `C_knots' , `T_knots' ) < 1E-8
_assert_streq `"`: rowfullnames `C_knots''"' `"grade"'
_assert_streq `"`: colfullnames `C_knots''"' `"knot1 knot2 knot3 knot4 knot5"'
mat drop `C_knots' `T_knots'

sum sp1
assert         r(sum)   == 29394
assert         r(max)   == 18
assert         r(min)   == 4
assert reldif( r(sd)     , 2.491804267965819 ) <  1E-8
assert reldif( r(Var)    , 6.209088509852673 ) <  1E-8
assert reldif( r(mean)   , 13.11061552185549 ) <  1E-8
assert         r(sum_w) == 2242
assert         r(N)     == 2242

sum sp2
assert reldif( r(sum)    , 3711.796357750893 ) <  1E-8
assert reldif( r(max)    , 7.222222328186035 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , 2.1683050954753   ) <  1E-8
assert reldif( r(Var)    , 4.701546987064152 ) <  1E-8
assert reldif( r(mean)   , 1.655573754572209 ) <  1E-8
assert         r(sum_w) == 2242
assert         r(N)     == 2242

sum sp3
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4396680399363299 ) <  1E-8
assert reldif( r(Var)    , .1933079853414542 ) <  1E-8
assert reldif( r(mean)   , .2453662356183956 ) <  1E-8
assert         r(sum_w) == 2242
assert         r(N)     == 2242

sum sp4
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4396680399363299 ) <  1E-8
assert reldif( r(Var)    , .1933079853414542 ) <  1E-8
assert reldif( r(mean)   , .2453662356183956 ) <  1E-8
assert         r(sum_w) == 2242
assert         r(N)     == 2242

sum pr1
assert reldif( r(sum)    , 1440.000019341707 ) <  1E-8
assert reldif( r(max)    , .7189269661903381 ) <  1E-8
assert reldif( r(min)    , .3394427299499512 ) <  1E-8
assert reldif( r(sd)     , .1067762694174826 ) <  1E-8
assert reldif( r(Var)    , .0114011717107148 ) <  1E-8
assert reldif( r(mean)   , .6422836839169078 ) <  1E-8
assert         r(sum_w) == 2242
assert         r(N)     == 2242

sum pr2
assert reldif( r(sum)    , 233.0000031031668 ) <  1E-8
assert reldif( r(max)    , .2125695645809174 ) <  1E-8
assert reldif( r(min)    , .049348421394825  ) <  1E-8
assert reldif( r(sd)     , .0612599149925312 ) <  1E-8
assert reldif( r(Var)    , .0037527771848922 ) <  1E-8
assert reldif( r(mean)   , .103925068288656  ) <  1E-8
assert         r(sum_w) == 2242
assert         r(N)     == 2242

sum pr3
assert reldif( r(sum)    , 568.9999962896109 ) <  1E-8
assert reldif( r(max)    , .5591265559196472 ) <  1E-8
assert reldif( r(min)    , .1569962799549103 ) <  1E-8
assert reldif( r(sd)     , .0653069072736716 ) <  1E-8
assert reldif( r(Var)    , .0042649921376519 ) <  1E-8
assert reldif( r(mean)   , .2537912561505847 ) <  1E-8
assert         r(sum_w) == 2242
assert         r(N)     == 2242

keep if grade > 3
proprcspline marst grade, xlab(0(5)15)  by(black, note("")) showknots gen(pr_r)
assert pr1 == pr_r1
assert pr2 == pr_r2
assert pr3 == pr_r3

use nlsw88, clear
gen marst = cond(married, 1, ///
            cond(never_married, 2, 3)) if !missing(married, never_married)
label define marst 1 "married" ///
                   2 "never married" ///
                   3 "divorced/widowed"
label value marst marst

// cataxis is not allowed with more than 2 categories in by()
proprcspline marst grade, xlab(0(5)15)  by(race, note(""))
rcof `"noisily proprcspline marst grade, xlab(0(5)15)  by(race, note("")) cataxis"' == 198


use nlsw88, clear
gen marst = cond(married, 1, ///
            cond(never_married, 2, 3)) if !missing(married, never_married)
label define marst 1 "married" ///
                   2 "never married" ///
                   3 "divorced/widowed"
label value marst marst

gen black = race == 2 if race < .
label define black 1 "black" ///
                   0 "non-black"
label value black black


proprcspline marst grade c_city, xlab(0(5)15)  by(black, note("")) showknots stub(sp) gen(pr)

assert         r(N_knots) == 5

tempname T_knots 
mat `T_knots' = J(1,5,0)
mat `T_knots'[1,1] =                  9
mat `T_knots'[1,2] =                 12
mat `T_knots'[1,3] =                 12
mat `T_knots'[1,4] =                 14
mat `T_knots'[1,5] =                 18
tempname C_knots
matrix `C_knots' = r(knots)
assert mreldif( `C_knots' , `T_knots' ) < 1E-8
_assert_streq `"`: rowfullnames `C_knots''"' `"grade"'
_assert_streq `"`: colfullnames `C_knots''"' `"knot1 knot2 knot3 knot4 knot5"'
mat drop `C_knots' `T_knots'

sum sp1
assert         r(sum)   == 29394
assert         r(max)   == 18
assert         r(min)   == 0
assert reldif( r(sd)     , 2.521246094581113 ) <  1E-8
assert reldif( r(Var)    , 6.356681869440517 ) <  1E-8
assert reldif( r(mean)   , 13.09893048128342 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp2
assert reldif( r(sum)    , 3711.796357750893 ) <  1E-8
assert reldif( r(max)    , 7.222222328186035 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , 2.167901426346746 ) <  1E-8
assert reldif( r(Var)    , 4.699796594356254 ) <  1E-8
assert reldif( r(mean)   , 1.654098198641218 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp3
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4395329954959572 ) <  1E-8
assert reldif( r(Var)    , .1931892541296491 ) <  1E-8
assert reldif( r(mean)   , .2451475491338873 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp4
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4395329954959572 ) <  1E-8
assert reldif( r(Var)    , .1931892541296491 ) <  1E-8
assert reldif( r(mean)   , .2451475491338873 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr1
assert reldif( r(sum)    , 1450.052878946066 ) <  1E-8
assert reldif( r(max)    , .7135162949562073 ) <  1E-8
assert reldif( r(min)    , .2225368618965149 ) <  1E-8
assert reldif( r(sd)     , .0925085813552902 ) <  1E-8
assert reldif( r(Var)    , .0085578376243683 ) <  1E-8
assert reldif( r(mean)   , .6461911225249848 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr2
assert reldif( r(sum)    , 218.4000144153833 ) <  1E-8
assert reldif( r(max)    , .2079542726278305 ) <  1E-8
assert reldif( r(min)    , .051213875412941  ) <  1E-8
assert reldif( r(sd)     , .05156975198903   ) <  1E-8
assert reldif( r(Var)    , .0026594393202101 ) <  1E-8
assert reldif( r(mean)   , .0973262096325238 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr3
assert reldif( r(sum)    , 575.5471099019051 ) <  1E-8
assert reldif( r(max)    , .5890390276908875 ) <  1E-8
assert reldif( r(min)    , .1572178155183792 ) <  1E-8
assert reldif( r(sd)     , .0626419886238584 ) <  1E-8
assert reldif( r(Var)    , .0039240187387516 ) <  1E-8
assert reldif( r(mean)   , .2564826692967491 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

drop  sp1- pr3
proprcspline marst grade c_city age, xlab(0(5)15)  by(black, note("")) showknots stub(sp) gen(pr) at(c_city 1)

assert         r(N_knots) == 5

tempname T_knots 
mat `T_knots' = J(1,5,0)
mat `T_knots'[1,1] =                  9
mat `T_knots'[1,2] =                 12
mat `T_knots'[1,3] =                 12
mat `T_knots'[1,4] =                 14
mat `T_knots'[1,5] =                 18
tempname C_knots
matrix `C_knots' = r(knots)
assert mreldif( `C_knots' , `T_knots' ) < 1E-8
_assert_streq `"`: rowfullnames `C_knots''"' `"grade"'
_assert_streq `"`: colfullnames `C_knots''"' `"knot1 knot2 knot3 knot4 knot5"'
mat drop `C_knots' `T_knots'

sum sp1
assert         r(sum)   == 29394
assert         r(max)   == 18
assert         r(min)   == 0
assert reldif( r(sd)     , 2.521246094581113 ) <  1E-8
assert reldif( r(Var)    , 6.356681869440517 ) <  1E-8
assert reldif( r(mean)   , 13.09893048128342 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp2
assert reldif( r(sum)    , 3711.796357750893 ) <  1E-8
assert reldif( r(max)    , 7.222222328186035 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , 2.167901426346746 ) <  1E-8
assert reldif( r(Var)    , 4.699796594356254 ) <  1E-8
assert reldif( r(mean)   , 1.654098198641218 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp3
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4395329954959572 ) <  1E-8
assert reldif( r(Var)    , .1931892541296491 ) <  1E-8
assert reldif( r(mean)   , .2451475491338873 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum sp4
assert reldif( r(sum)    , 550.111100256443  ) <  1E-8
assert reldif( r(max)    , 1.481481432914734 ) <  1E-8
assert         r(min)   == 0
assert reldif( r(sd)     , .4395329954959572 ) <  1E-8
assert reldif( r(Var)    , .1931892541296491 ) <  1E-8
assert reldif( r(mean)   , .2451475491338873 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr1
assert reldif( r(sum)    , 1260.41586278379  ) <  1E-8
assert reldif( r(max)    , .6374779939651489 ) <  1E-8
assert reldif( r(min)    , .1736584156751633 ) <  1E-8
assert reldif( r(sd)     , .1036590576630381 ) <  1E-8
assert reldif( r(Var)    , .0107452002355891 ) <  1E-8
assert reldif( r(mean)   , .5616826482993715 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr2
assert reldif( r(sum)    , 316.0454156324267 ) <  1E-8
assert reldif( r(max)    , .3170115351676941 ) <  1E-8
assert reldif( r(min)    , .0709226951003075 ) <  1E-8
assert reldif( r(sd)     , .0787768201777158 ) <  1E-8
assert reldif( r(Var)    , .0062057873973122 ) <  1E-8
assert reldif( r(mean)   , .1408402030447535 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

sum pr3
assert reldif( r(sum)    , 667.538722127676  ) <  1E-8
assert reldif( r(max)    , .5093300342559815 ) <  1E-8
assert reldif( r(min)    , .1787750869989395 ) <  1E-8
assert reldif( r(sd)     , .0545128408261871 ) <  1E-8
assert reldif( r(Var)    , .0029716498149412 ) <  1E-8
assert reldif( r(mean)   , .2974771488982513 ) <  1E-8
assert         r(sum_w) == 2244
assert         r(N)     == 2244

drop sp1-pr3

// this used to give a problem witht the second y-axis labels
drop if black == 1 & grade ==18
proprcspline marst grade, xlab(0(5)15)  by(black, note("")) 
